from .exception import BasicException


class UnProcessableEntityException(BasicException):

    def __init__(self, message, error_code=None):
        BasicException.__init__(self)
        self.status_code = 422
        self.error_code = error_code
        self.message = message



class InvalidRequestObject:

    def __init__(self):
        self.errors = []

    def add_error(self, parameter, message):
        self.errors.append({'parameter': parameter, 'message': message})

    def has_errors(self):
        return len(self.errors) > 0

    def __nonzero__(self):
        return False

    __bool__ = __nonzero__


class ValidRequestObject:

    @classmethod
    def from_dict(cls, adict):
        raise NotImplementedError

    def __nonzero__(self):
        return True

    __bool__ = __nonzero__



import abc
from enum import Enum
from datetime import datetime


class Entity:

    @classmethod
    @abc.abstractmethod
    def from_dict(cls, adict):
        pass

    def entity2dict(self, entity):
        def _type_switcher(value):
            if isinstance(value, Entity):
                return self.entity2dict(value)
            elif isinstance(value, Enum):
                return value.name
            elif isinstance(value, dict):
                return value
            elif isinstance(value, datetime):
                return str(value)
            elif isinstance(value, list):
                if len(value) == 0:
                    return []
                else:
                    if all(isinstance(e, Entity) for e in value):
                        return [self.entity2dict(e) for e in value]
                    else:
                        return value
            else:
                return value

        if isinstance(entity, list):
            if all(isinstance(elem, Entity) for elem in entity):
                return [self.entity2dict(elem) for elem in entity]
            else:
                return entity

        if not isinstance(entity, Entity):
            return entity

        to_serialize = vars(entity)
        return {key: _type_switcher(value) for key, value in to_serialize.items() if value is not None}

    def to_dict(self):
        return self.entity2dict(self)

    def __eq__(self, other):
        return self.to_dict() == other.to_dict()


import json


class Serializer(json.JSONEncoder):
    # TODO(jay): added except for schema errors
    def default(self, o):
        try:
            return self.schema.validate(o.to_dict())
        except AttributeError:
            return super().default(o)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="clean-architecture",
    version="0.0.3",
    author="jaystevency",
    author_email="yjy1129@kookmin.ac.kr",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/JayStevency/clean-architecture",
    download_url="https://github.com/JayStevency/clean-architecture",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)



from .exception import BasicException
from .bad_request_exception import BadRequestException
from .conflicted_exception import ConflictedException
from .forbidden_exception import ForbiddenException
from .method_not_allowed_exception import MethodNotAllowedException
from .not_acceptable_exception import NotAcceptableException
from .payment_required_exception import PaymentRequiredException
from .precondition_failed_exception import PreconditionFailedException
from .unauthorized_exception import UnauthorizedException
from .unprocessable_entity_exception import UnProcessableEntityException
from .not_found_exception import NotFoundException

__all__ = [
    'BasicException',
    'BadRequestException',
    'ConflictedException',
    'ForbiddenException',
    'MethodNotAllowedException',
    'NotAcceptableException',
    'PaymentRequiredException',
    'PreconditionFailedException',
    'UnauthorizedException',
    'UnProcessableEntityException',
    'NotFoundException'
]


from clean_architecture.exception import (
    BadRequestException,
    ConflictedException,
    ForbiddenException,
    MethodNotAllowedException,
    NotAcceptableException,
    NotFoundException,
    PaymentRequiredException,
    PreconditionFailedException,
    UnauthorizedException,
    UnProcessableEntityException,
)

from .response_object import ResponseFailure


class UseCase:
    def execute(self, request_object):
        if not request_object:
            return ResponseFailure.build_from_invalid_request_object(request_object)
        try:
            return self.process_request(request_object)
        except BadRequestException as e:
            return ResponseFailure.build_resource_error(
                status_code=e.status_code, error_code=e.error_code, message=e.message
            )
        except ConflictedException as e:
            return ResponseFailure.build_resource_error(
                status_code=e.status_code, error_code=e.error_code, message=e.message
            )
        except ForbiddenException as e:
            return ResponseFailure.build_resource_error(
                status_code=e.status_code, error_code=e.error_code, message=e.message
            )
        except MethodNotAllowedException as e:
            return ResponseFailure.build_resource_error(
                status_code=e.status_code, error_code=e.error_code, message=e.message
            )
        except NotAcceptableException as e:
            return ResponseFailure.build_resource_error(
                status_code=e.status_code, error_code=e.error_code, message=e.message
            )
        except NotFoundException as e:
            return ResponseFailure.build_resource_error(
                status_code=e.status_code, error_code=e.error_code, message=e.message
            )
        except PaymentRequiredException as e:
            return ResponseFailure.build_resource_error(
                status_code=e.status_code, error_code=e.error_code, message=e.message
            )
        except PreconditionFailedException as e:
            return ResponseFailure.build_resource_error(
                status_code=e.status_code, error_code=e.error_code, message=e.message
            )
        except UnauthorizedException as e:
            return ResponseFailure.build_resource_error(
                status_code=e.status_code, error_code=e.error_code, message=e.message
            )
        except UnProcessableEntityException as e:
            return ResponseFailure.build_resource_error(
                status_code=e.status_code, error_code=e.error_code, message=e.message
            )
        except Exception as exc:
            return ResponseFailure.build_system_error(
                message="{}: {}".format(exc.__class__.__name__, "{}".format(exc)))

    def process_request(self, request_object):
        raise NotImplementedError(
            "process_request() not implemented by UseCase class"
        )



from .request_object import InvalidRequestObject, ValidRequestObject
from .response_object import ResponseFailure, ResponseSuccess
from .use_case import UseCase


__all__ = [
    'InvalidRequestObject',
    'ValidRequestObject',
    'ResponseSuccess',
    'ResponseFailure',
    'UseCase'
]


class ResponseSuccess:

    def __init__(self, type_, value=None):
        self.type = type_
        self.value = value

    def __nonzero__(self):
        return True

    __bool__ = __nonzero__


class ResponseFailure:

    def __init__(self, type_, message, error_code):
        self.type = type_
        self.error_code = error_code
        self.message = self._format_message(message)

    def _format_message(self, msg):
        if isinstance(msg, Exception):
            return "{}: {}".format(msg.__class__.__name__, "{}".format(msg))
        return msg

    @property
    def value(self):
        return {'type': self.type, 'error_code': self.error_code, 'message': self.message}

    def __bool__(self):
        return False

    @classmethod
    def build_resource_error(cls, status_code=404, message=None, error_code=None,):
        return cls(status_code, message, error_code)

    @classmethod
    def build_system_error(cls, status_code=500, message=None, error_code=None):
        return cls(status_code, message, error_code)

    @classmethod
    def build_parameters_error(cls, status_code=400, message=None, error_code=None):
        return cls(status_code, message, error_code)

    @classmethod
    def build_from_invalid_request_object(cls, invalid_request_object):
        message = "\n".join(["{}: {}".format(err['parameter'], err['message'])
                             for err in invalid_request_object.errors])
        return cls.build_parameters_error(message=message)
